@@ -1,6 +1,8 @@
 import warnings
 import numpy as np
 import datetime
+import json
+import hashlib
 from mugatu.exceptions import MugatuError, MugatuWarning
 
 try:
@@ -100,6 +102,117 @@ def make_design_field_targetdb(cadence, fieldid, plan,
         fieldDB.save()
 
 
+def make_desigmmode_results_targetdb(design_id, design_pass,
+                                     design_valid_file_row=None,
+                                     design_status=None,
+                                     boss_skies_min_pass=None,
+                                     boss_skies_min_value=None,
+                                     boss_skies_fov_pass=None,
+                                     boss_skies_fov_value=None,
+                                     apogee_skies_min_pass=None,
+                                     apogee_skies_min_value=None,
+                                     apogee_skies_fov_pass=None,
+                                     apogee_skies_fov_value=None,
+                                     boss_stds_min_pass=None,
+                                     boss_stds_min_value=None,
+                                     boss_stds_fov_pass=None,
+                                     boss_stds_fov_value=None,
+                                     apogee_stds_min_pass=None,
+                                     apogee_stds_min_value=None,
+                                     apogee_stds_fov_pass=None,
+                                     apogee_stds_fov_value=None,
+                                     boss_stds_mags_pass=None,
+                                     apogee_stds_mags_pass=None,
+                                     boss_bright_limit_targets_pass=None,
+                                     apogee_bright_limit_targets_pass=None,
+                                     boss_sky_neighbors_targets_pass=None,
+                                     apogee_sky_neighbors_targets_pass=None,
+                                     apogee_trace_diff_targets_pass=None):
+    """
+    add designmode check results to targetdb
+    """
+    if design_valid_file_row is not None:
+        design_status = None
+        boss_skies_min_pass = design_valid_file_row['boss_n_skies_min']
+        boss_skies_min_value = design_valid_file_row['boss_n_skies_min_value']
+        boss_skies_fov_pass = design_valid_file_row['boss_min_skies_fovmetric']
+        boss_skies_fov_value = design_valid_file_row['boss_min_skies_fovmetric_value']
+        apogee_skies_min_pass = design_valid_file_row['apogee_n_skies_min']
+        apogee_skies_min_value = design_valid_file_row['apogee_n_skies_min_value']
+        apogee_skies_fov_pass = design_valid_file_row['apogee_min_skies_fovmetric']
+        apogee_skies_fov_value = design_valid_file_row['apogee_min_skies_fovmetric_value']
+        boss_stds_min_pass = design_valid_file_row['boss_n_stds_min']
+        boss_stds_min_value = design_valid_file_row['boss_n_stds_min_value']
+        boss_stds_fov_pass = design_valid_file_row['boss_min_stds_fovmetric']
+        boss_stds_fov_value = design_valid_file_row['boss_min_stds_fovmetric_value']
+        apogee_stds_min_pass = design_valid_file_row['apogee_n_stds_min']
+        apogee_stds_min_value = design_valid_file_row['apogee_n_stds_min_value']
+        apogee_stds_fov_pass = design_valid_file_row['apogee_min_stds_fovmetric']
+        apogee_stds_fov_value = design_valid_file_row['apogee_min_stds_fovmetric_value']
+        boss_stds_mags_pass = design_valid_file_row['boss_stds_mags']
+        apogee_stds_mags_pass = design_valid_file_row['apogee_stds_mags']
+        boss_bright_limit_targets_pass = design_valid_file_row['boss_bright_limit_targets']
+        apogee_bright_limit_targets_pass = design_valid_file_row['apogee_bright_limit_targets']
+        boss_sky_neighbors_targets_pass = design_valid_file_row['boss_sky_neighbors_targets']
+        apogee_sky_neighbors_targets_pass = design_valid_file_row['apogee_sky_neighbors_targets']
+        apogee_trace_diff_targets_pass = None  # not checking currently
+    # write to database
+    design_checkDB = targetdb.DesignModeCheckResults.create(
+        design=design_id,
+        design_pass=design_pass,
+        design_status=design_status,
+        boss_skies_min_pass=boss_skies_min_pass,
+        boss_skies_min_value=boss_skies_min_value,
+        boss_skies_fov_pass=boss_skies_fov_pass,
+        boss_skies_fov_value=boss_skies_fov_value,
+        apogee_skies_min_pass=apogee_skies_min_pass,
+        apogee_skies_min_value=apogee_skies_min_value,
+        apogee_skies_fov_pass=apogee_skies_fov_pass,
+        apogee_skies_fov_value=apogee_skies_fov_value,
+        boss_stds_min_pass=boss_stds_min_pass,
+        boss_stds_min_value=boss_stds_min_value,
+        boss_stds_fov_pass=boss_stds_fov_pass,
+        boss_stds_fov_value=boss_stds_fov_value,
+        apogee_stds_min_pass=apogee_stds_min_pass,
+        apogee_stds_min_value=apogee_stds_min_value,
+        apogee_stds_fov_pass=apogee_stds_fov_pass,
+        apogee_stds_fov_value=apogee_stds_fov_value,
+        boss_stds_mags_pass=boss_stds_mags_pass,
+        apogee_stds_mags_pass=apogee_stds_mags_pass,
+        boss_bright_limit_targets_pass=boss_bright_limit_targets_pass,
+        apogee_bright_limit_targets_pass=apogee_bright_limit_targets_pass,
+        boss_sky_neighbors_targets_pass=boss_sky_neighbors_targets_pass,
+        apogee_sky_neighbors_targets_pass=apogee_sky_neighbors_targets_pass,
+        apogee_trace_diff_targets_pass=apogee_trace_diff_targets_pass)
+    design_checkDB.save()
+
+
+def assignment_hash(ids, holeIDs):
+    """
+    Create a unique hash for the design
+
+    Parameters
+    ----------
+    ids: np.array
+        carton_to_target_pks for the assignments.
+
+    holeIDs: np.array
+        corresponding holeIDs for the assignments.
+
+    Returns
+    -------
+    assign_hash: str
+        The assignment_hash for the design
+    """
+    assign_dict = {}
+    idx = np.argsort(holeIDs)
+    assign_dict['carton_to_target'] = ids[idx].tolist()
+    assign_dict['hole_id'] = holeIDs[idx].tolist()
+    assign_hash = (hashlib.md5(json.dumps(assign_dict)
+                               .encode('utf-8')).hexdigest())
+    return assign_hash
+
+
 def make_design_assignments_targetdb(plan, fieldid, exposure,
                                      desmode_label,
                                      design_ids, robotID, holeID,
@@ -216,11 +329,15 @@ def make_design_assignments_targetdb(plan, fieldid, exposure,
     else:
         fieldpk = fieldid[0].pk
 
+    assign_hash = assignment_hash(design_ids[robotID != -1],
+                                  holeID[robotID != -1])
+
     designDB = targetdb.Design.create(field=fieldpk,
                                       exposure=exposure,
                                       design_mode=desmode_label,
                                       mugatu_version=mugatu_version,
-                                      run_on=datetime.datetime.now())
+                                      run_on=datetime.datetime.now(),
+                                      assignment_hash=assign_hash)
     # save row
     designDB.save()
 
@@ -292,6 +409,9 @@ class TargetdbFieldIDs(object):
                  version_plan=None):
         self.fieldid_type = fieldid_type
         self.version_plan = version_plan
+        # get the reserved field_ids
+        field_reserve = targetdb.FieldReservation.select()
+        self.field_reserve = [f.field_id for f in field_reserve]
 
     def find_next_available(self):
         """
@@ -323,10 +443,27 @@ class TargetdbFieldIDs(object):
             .where(arg))
         if len(field_start) > 0:
             fieldids = [f.field_id for f in field_start]
+            # add reserved fieldids to list
+            other_reserved = list(set(self.field_reserve) - set(fieldids))
+            for oth in other_reserved:
+                if fieldid_bounds[1] == -999:
+                    if oth >= fieldid_bounds[0]:
+                        fieldids.append(oth)
+                else:
+                    if oth >= fieldid_bounds[0] and oth < fieldid_bounds[1]:
+                        fieldids.append(oth)
+            # sort in order
+            fieldids.sort()
+            # look for breaks
+            stop = False
             for i in range(1, len(fieldids)):
                 if fieldids[i] != fieldids[i - 1] + 1:
                     fieldid = fieldids[i - 1] + 1
+                    stop = True
                     break
+            # if no stop, then max + 1
+            if not stop:
+                fieldid = max(fieldids) + 1
         else:
             fieldid = fieldid_bounds[0]
         return fieldid
@@ -355,8 +492,12 @@ class TargetdbFieldIDs(object):
             arg = arg & (targetdb.Version.plan == self.version_plan)
         field = (targetdb.Field.select(
             targetdb.Field.field_id)
+            .join(targetdb.Version)
             .where(arg))
         fieldids = [f[0] for f in field.tuples()]
+        # add reserved fieldids to list
+        other_reserved = list(set(self.field_reserve) - set(fieldids))
+        fieldids += other_reserved
         if isinstance(fieldid, int):
             field_avail = fieldid not in fieldids
         else:
